@@ -1,7 +1,13 @@
 import numpy as np
 
-from pandas import DataFrame, Series, date_range, factorize, read_csv
-from pandas.core.algorithms import take_1d
+from pandas import (
+    DataFrame,
+    Series,
+    date_range,
+    factorize,
+    read_csv,
+)
+from pandas.core.algorithms import take_nd
 
 from .pandas_vb_common import tm
 
@@ -25,7 +31,7 @@ try:
 except ImportError:
     from pandas import algos
 try:
-    from pandas._testing import test_parallel
+    from pandas._testing import test_parallel  # noqa: PDF014
 
     have_real_test_parallel = True
 except ImportError:
@@ -110,7 +116,7 @@ class ParallelTake1D:
 
         @test_parallel(num_threads=2)
         def parallel_take1d():
-            take_1d(df["col"].values, indexer)
+            take_nd(df["col"].values, indexer)
 
         self.parallel_take1d = parallel_take1d
 
@@ -119,6 +125,7 @@ class ParallelTake1D:
 
 
 class ParallelKth:
+    # This depends exclusively on code in _libs/, could go in libs.py
 
     number = 1
     repeat = 5
