@@ -2,9 +2,19 @@ import sys
 
 import numpy as np
 
-from pandas import DataFrame, concat, date_range, read_json, timedelta_range
+from pandas import (
+    DataFrame,
+    concat,
+    date_range,
+    json_normalize,
+    read_json,
+    timedelta_range,
+)
 
-from ..pandas_vb_common import BaseIO, tm
+from ..pandas_vb_common import (
+    BaseIO,
+    tm,
+)
 
 
 class ReadJSON(BaseIO):
@@ -68,6 +78,27 @@ class ReadJSONLines(BaseIO):
         read_json(self.fname, orient="records", lines=True, nrows=15000)
 
 
+class NormalizeJSON(BaseIO):
+    fname = "__test__.json"
+    params = [
+        ["split", "columns", "index", "values", "records"],
+        ["df", "df_date_idx", "df_td_int_ts", "df_int_floats", "df_int_float_str"],
+    ]
+    param_names = ["orient", "frame"]
+
+    def setup(self, orient, frame):
+        data = {
+            "hello": ["thisisatest", 999898, "mixed types"],
+            "nest1": {"nest2": {"nest3": "nest3_value", "nest3_int": 3445}},
+            "nest1_list": {"nest2": ["blah", 32423, 546456.876, 92030234]},
+            "hello2": "string",
+        }
+        self.data = [data for i in range(10000)]
+
+    def time_normalize_json(self, orient, frame):
+        json_normalize(self.data)
+
+
 class ToJSON(BaseIO):
 
     fname = "__test__.json"
