@@ -1,5 +1,8 @@
 try:
-    from pandas._libs.tslibs import is_date_array_normalized, normalize_i8_timestamps
+    from pandas._libs.tslibs import (
+        is_date_array_normalized,
+        normalize_i8_timestamps,
+    )
 except ImportError:
     from pandas._libs.tslibs.conversion import (
         normalize_i8_timestamps,
@@ -8,7 +11,11 @@ except ImportError:
 
 import pandas as pd
 
-from .tslib import _sizes, _tzs
+from .tslib import (
+    _sizes,
+    _tzs,
+    tzlocal_obj,
+)
 
 
 class Normalize:
@@ -24,6 +31,10 @@ class Normalize:
         dti = pd.date_range("2016-01-01", periods=10, tz=tz).repeat(size // 10)
         self.i8data = dti.asi8
 
+        if size == 10 ** 6 and tz is tzlocal_obj:
+            # tzlocal is cumbersomely slow, so skip to keep runtime in check
+            raise NotImplementedError
+
     def time_normalize_i8_timestamps(self, size, tz):
         normalize_i8_timestamps(self.i8data, tz)
 
