@@ -3,7 +3,11 @@ from pytz import UTC
 
 from pandas._libs.tslibs.tzconversion import tz_localize_to_utc
 
-from .tslib import _sizes, _tzs
+from .tslib import (
+    _sizes,
+    _tzs,
+    tzlocal_obj,
+)
 
 try:
     old_sig = False
@@ -21,6 +25,10 @@ class TimeTZConvert:
     param_names = ["size", "tz"]
 
     def setup(self, size, tz):
+        if size == 10 ** 6 and tz is tzlocal_obj:
+            # tzlocal is cumbersomely slow, so skip to keep runtime in check
+            raise NotImplementedError
+
         arr = np.random.randint(0, 10, size=size, dtype="i8")
         self.i8data = arr
 
@@ -28,9 +36,6 @@ class TimeTZConvert:
         # effectively:
         #  dti = DatetimeIndex(self.i8data, tz=tz)
         #  dti.tz_localize(None)
-        if size >= 10 ** 6 and str(tz) == "tzlocal()":
-            # asv fill will because each call takes 8+seconds
-            return
         if old_sig:
             tz_convert_from_utc(self.i8data, UTC, tz)
         else:
