@@ -728,20 +728,19 @@ INLINE_PREFIX void FASTCALL_MSVC strreverse(char *begin,
     while (end > begin) aux = *end, *end-- = *begin, *begin++ = aux;
 }
 
-void Buffer_AppendIndentNewlineUnchecked(JSONObjectEncoder *enc)
-{
+void Buffer_AppendIndentNewlineUnchecked(JSONObjectEncoder *enc) {
   if (enc->indent > 0) Buffer_AppendCharUnchecked(enc, '\n');
 }
 
 // This function could be refactored to only accept enc as an argument,
 // but this is a straight vendor from ujson source
-void Buffer_AppendIndentUnchecked(JSONObjectEncoder *enc, JSINT32 value)
-{
+void Buffer_AppendIndentUnchecked(JSONObjectEncoder *enc, JSINT32 value) {
   int i;
-  if (enc->indent > 0)
+  if (enc->indent > 0) {
     while (value-- > 0)
       for (i = 0; i < enc->indent; i++)
         Buffer_AppendCharUnchecked(enc, ' ');
+  }
 }
 
 void Buffer_AppendIntUnchecked(JSONObjectEncoder *enc, JSINT32 value) {
@@ -976,7 +975,7 @@ void encode(JSOBJ obj, JSONObjectEncoder *enc, const char *name,
             enc->iterBegin(obj, &tc);
 
             Buffer_AppendCharUnchecked(enc, '[');
-            Buffer_AppendIndentNewlineUnchecked (enc);
+            Buffer_AppendIndentNewlineUnchecked(enc);
 
             while (enc->iterNext(obj, &tc)) {
                 if (count > 0) {
@@ -984,20 +983,20 @@ void encode(JSOBJ obj, JSONObjectEncoder *enc, const char *name,
 #ifndef JSON_NO_EXTRA_WHITESPACE
                     Buffer_AppendCharUnchecked(buffer, ' ');
 #endif
-                    Buffer_AppendIndentNewlineUnchecked (enc);
+                    Buffer_AppendIndentNewlineUnchecked(enc);
                 }
 
                 iterObj = enc->iterGetValue(obj, &tc);
 
                 enc->level++;
-                Buffer_AppendIndentUnchecked (enc, enc->level);
+                Buffer_AppendIndentUnchecked(enc, enc->level);
                 encode(iterObj, enc, NULL, 0);
                 count++;
             }
 
             enc->iterEnd(obj, &tc);
-            Buffer_AppendIndentNewlineUnchecked (enc);
-            Buffer_AppendIndentUnchecked (enc, enc->level);
+            Buffer_AppendIndentNewlineUnchecked(enc);
+            Buffer_AppendIndentUnchecked(enc, enc->level);
             Buffer_AppendCharUnchecked(enc, ']');
             break;
         }
@@ -1007,7 +1006,7 @@ void encode(JSOBJ obj, JSONObjectEncoder *enc, const char *name,
             enc->iterBegin(obj, &tc);
 
             Buffer_AppendCharUnchecked(enc, '{');
-            Buffer_AppendIndentNewlineUnchecked (enc);
+            Buffer_AppendIndentNewlineUnchecked(enc);
 
             while (enc->iterNext(obj, &tc)) {
                 if (count > 0) {
@@ -1015,21 +1014,21 @@ void encode(JSOBJ obj, JSONObjectEncoder *enc, const char *name,
 #ifndef JSON_NO_EXTRA_WHITESPACE
                     Buffer_AppendCharUnchecked(enc, ' ');
 #endif
-                    Buffer_AppendIndentNewlineUnchecked (enc);
+                    Buffer_AppendIndentNewlineUnchecked(enc);
                 }
 
                 iterObj = enc->iterGetValue(obj, &tc);
                 objName = enc->iterGetName(obj, &tc, &szlen);
 
                 enc->level++;
-                Buffer_AppendIndentUnchecked (enc, enc->level);
+                Buffer_AppendIndentUnchecked(enc, enc->level);
                 encode(iterObj, enc, objName, szlen);
                 count++;
             }
 
             enc->iterEnd(obj, &tc);
-            Buffer_AppendIndentNewlineUnchecked (enc);
-            Buffer_AppendIndentUnchecked (enc, enc->level);
+            Buffer_AppendIndentNewlineUnchecked(enc);
+            Buffer_AppendIndentUnchecked(enc, enc->level);
             Buffer_AppendCharUnchecked(enc, '}');
             break;
         }
@@ -1134,7 +1133,6 @@ void encode(JSOBJ obj, JSONObjectEncoder *enc, const char *name,
             }
 
             break;
-            
         }
     }
 
