@@ -1,5 +1,12 @@
-#ifndef PANDAS__LIBS_SRC_UJSON_DATE_CONVERSIONS
-#define PANDAS__LIBS_SRC_UJSON_DATE_CONVERSIONS
+/*
+Copyright (c) 2020, PyData Development Team
+All rights reserved.
+Distributed under the terms of the BSD Simplified License.
+The full license is in the LICENSE file, distributed with this software.
+*/
+
+#ifndef PANDAS__LIBS_SRC_UJSON_PYTHON_DATE_CONVERSIONS_H_
+#define PANDAS__LIBS_SRC_UJSON_PYTHON_DATE_CONVERSIONS_H_
 
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
@@ -14,8 +21,8 @@ int scaleNanosecToUnit(npy_int64 *value, NPY_DATETIMEUNIT unit);
 // len is mutated to save the length of the returned string
 char *int64ToIso(int64_t value, NPY_DATETIMEUNIT base, size_t *len);
 
-// TODO: this function doesn't do a lot; should augment or replace with
-// scaleNanosecToUnit
+// TODO(username): this function doesn't do a lot; should augment or
+// replace with scaleNanosecToUnit
 npy_datetime NpyDateTimeToEpoch(npy_datetime dt, NPY_DATETIMEUNIT base);
 
 // Converts a Python object representing a Date / Datetime to ISO format
@@ -29,4 +36,4 @@ npy_datetime PyDateTimeToEpoch(PyObject *dt, NPY_DATETIMEUNIT base);
 
 char *int64ToIsoDuration(int64_t value, size_t *len);
 
-#endif
+#endif  // PANDAS__LIBS_SRC_UJSON_PYTHON_DATE_CONVERSIONS_H_
