@@ -4,9 +4,11 @@ All of pandas' ExtensionArrays.
 See :ref:`extending.extension-types` for more.
 """
 from pandas.core.arrays import (
+    ArrowStringArray,
     BooleanArray,
     Categorical,
     DatetimeArray,
+    FloatingArray,
     IntegerArray,
     IntervalArray,
     PandasArray,
@@ -17,9 +19,11 @@ from pandas.core.arrays import (
 )
 
 __all__ = [
+    "ArrowStringArray",
     "BooleanArray",
     "Categorical",
     "DatetimeArray",
+    "FloatingArray",
     "IntegerArray",
     "IntervalArray",
     "PandasArray",
