@@ -1,15 +1,17 @@
 import numpy as np
 import pytest
 
-from pandas.compat import PY37
-
 import pandas as pd
 import pandas._testing as tm
+from pandas.api.types import is_bool_dtype
 from pandas.tests.extension import base
 
 pytest.importorskip("pyarrow", minversion="0.13.0")
 
-from .arrays import ArrowBoolArray, ArrowBoolDtype  # isort:skip
+from pandas.tests.extension.arrow.arrays import (  # isort:skip
+    ArrowBoolArray,
+    ArrowBoolDtype,
+)
 
 
 @pytest.fixture
@@ -52,6 +54,10 @@ class TestInterface(BaseArrowTests, base.BaseInterfaceTests):
         # __setitem__ does not work, so we only have a smoke-test
         data.view()
 
+    @pytest.mark.xfail(raises=AssertionError, reason="Not implemented yet")
+    def test_contains(self, data, data_missing):
+        super().test_contains(data, data_missing)
+
 
 class TestConstructors(BaseArrowTests, base.BaseConstructorsTests):
     def test_from_dtype(self, data):
@@ -62,13 +68,11 @@ class TestConstructors(BaseArrowTests, base.BaseConstructorsTests):
     def test_from_sequence_from_cls(self, data):
         super().test_from_sequence_from_cls(data)
 
-    @pytest.mark.skipif(not PY37, reason="timeout on Linux py36_locale")
     @pytest.mark.xfail(reason="pa.NULL is not recognised as scalar, GH-33899")
     def test_series_constructor_no_data_with_index(self, dtype, na_value):
         # pyarrow.lib.ArrowInvalid: only handle 1-dimensional arrays
         super().test_series_constructor_no_data_with_index(dtype, na_value)
 
-    @pytest.mark.skipif(not PY37, reason="timeout on Linux py36_locale")
     @pytest.mark.xfail(reason="pa.NULL is not recognised as scalar, GH-33899")
     def test_series_constructor_scalar_na_with_index(self, dtype, na_value):
         # pyarrow.lib.ArrowInvalid: only handle 1-dimensional arrays
@@ -78,6 +82,10 @@ class TestConstructors(BaseArrowTests, base.BaseConstructorsTests):
     def test_construct_empty_dataframe(self, dtype):
         super().test_construct_empty_dataframe(dtype)
 
+    @pytest.mark.xfail(reason="_from_sequence ignores dtype keyword")
+    def test_empty(self, dtype):
+        super().test_empty(dtype)
+
 
 class TestReduce(base.BaseNoReduceTests):
     def test_reduce_series_boolean(self):
@@ -89,7 +97,7 @@ class TestReduceBoolean(base.BaseBooleanReduceTests):
 
 
 def test_is_bool_dtype(data):
-    assert pd.api.types.is_bool_dtype(data)
+    assert is_bool_dtype(data)
     assert pd.core.common.is_bool_indexer(data)
     s = pd.Series(range(len(data)))
     result = s[data]
