@@ -1,8 +1,11 @@
 import pytest
 
 import pandas as pd
-
-from .array import ListArray, ListDtype, make_data
+from pandas.tests.extension.list.array import (
+    ListArray,
+    ListDtype,
+    make_data,
+)
 
 
 @pytest.fixture
