@@ -1,24 +1,24 @@
 import numpy as np
 import pytest
 
+from pandas._libs.internals import BlockPlacement
+import pandas.util._test_decorators as td
+
 import pandas as pd
 from pandas.core.internals import BlockManager
 from pandas.core.internals.blocks import ExtensionBlock
 
+pytestmark = td.skip_array_manager_invalid_test
+
 
 class CustomBlock(ExtensionBlock):
 
     _holder = np.ndarray
-    _can_hold_na = False
 
-    def concat_same_type(self, to_concat, placement=None):
-        """
-        Always concatenate disregarding self.ndim as the values are
-        always 1D in this custom Block
-        """
-        values = np.concatenate([blk.values for blk in to_concat])
-        placement = self.mgr_locs if self.ndim == 2 else slice(len(values))
-        return self.make_block_same_class(values, placement=placement)
+    # Cannot override final attribute "_can_hold_na"
+    @property  # type: ignore[misc]
+    def _can_hold_na(self) -> bool:
+        return False
 
 
 @pytest.fixture
@@ -26,7 +26,8 @@ def df():
     df1 = pd.DataFrame({"a": [1, 2, 3]})
     blocks = df1._mgr.blocks
     values = np.arange(3, dtype="int64")
-    custom_block = CustomBlock(values, placement=slice(1, 2))
+    bp = BlockPlacement(slice(1, 2))
+    custom_block = CustomBlock(values, placement=bp, ndim=2)
     blocks = blocks + (custom_block,)
     block_manager = BlockManager(blocks, [pd.Index(["a", "b"]), df1.index])
     return pd.DataFrame(block_manager)
