@@ -1,11 +1,15 @@
+from __future__ import annotations
+
 from datetime import timedelta
 import itertools
-from typing import Dict, List
 
 import numpy as np
 import pytest
 
-import pandas.compat as compat
+from pandas.compat import (
+    IS64,
+    is_platform_windows,
+)
 
 import pandas as pd
 import pandas._testing as tm
@@ -29,11 +33,21 @@ def check_comprehensiveness(request):
             klass in x.name and dtype in x.name and method in x.name for x in cls_funcs
         )
 
-    for combo in combos:
-        if not has_test(combo):
-            raise AssertionError(f"test method is not defined: {cls.__name__}, {combo}")
+    opts = request.config.option
+    if opts.lf or opts.keyword:
+        # If we are running with "last-failed" or -k foo, we expect to only
+        #  run a subset of tests.
+        yield
 
-    yield
+    else:
+
+        for combo in combos:
+            if not has_test(combo):
+                raise AssertionError(
+                    f"test method is not defined: {cls.__name__}, {combo}"
+                )
+
+        yield
 
 
 class CoercionBase:
@@ -55,17 +69,6 @@ class CoercionBase:
     def method(self):
         raise NotImplementedError(self)
 
-    def _assert(self, left, right, dtype):
-        # explicitly check dtype to avoid any unexpected result
-        if isinstance(left, pd.Series):
-            tm.assert_series_equal(left, right)
-        elif isinstance(left, pd.Index):
-            tm.assert_index_equal(left, right)
-        else:
-            raise NotImplementedError
-        assert left.dtype == dtype
-        assert right.dtype == dtype
-
 
 class TestSetitemCoercion(CoercionBase):
 
@@ -74,20 +77,21 @@ class TestSetitemCoercion(CoercionBase):
     def _assert_setitem_series_conversion(
         self, original_series, loc_value, expected_series, expected_dtype
     ):
-        """ test series value's coercion triggered by assignment """
+        """test series value's coercion triggered by assignment"""
         temp = original_series.copy()
         temp[1] = loc_value
         tm.assert_series_equal(temp, expected_series)
         # check dtype explicitly for sure
         assert temp.dtype == expected_dtype
 
+        # FIXME: dont leave commented-out
         # .loc works different rule, temporary disable
         # temp = original_series.copy()
         # temp.loc[1] = loc_value
         # tm.assert_series_equal(temp, expected_series)
 
     @pytest.mark.parametrize(
-        "val,exp_dtype", [(1, object), (1.1, object), (1 + 1j, object), (True, object)],
+        "val,exp_dtype", [(1, object), (1.1, object), (1 + 1j, object), (True, object)]
     )
     def test_setitem_series_object(self, val, exp_dtype):
         obj = pd.Series(list("abcd"))
@@ -124,11 +128,14 @@ class TestSetitemCoercion(CoercionBase):
             exp = pd.Series([1, 0, 3, 4], dtype=np.int8)
             self._assert_setitem_series_conversion(obj, val, exp, np.int8)
             mark = pytest.mark.xfail(
-                reason="BUG: it must be Series([1, 1, 3, 4], dtype=np.int16"
+                reason="BUG: it must be pd.Series([1, 1, 3, 4], dtype=np.int16"
             )
             request.node.add_marker(mark)
 
-        exp = pd.Series([1, val, 3, 4], dtype=np.int8)
+        warn = None if exp_dtype is np.int8 else FutureWarning
+        msg = "Values are too large to be losslessly cast to int8"
+        with tm.assert_produces_warning(warn, match=msg):
+            exp = pd.Series([1, val, 3, 4], dtype=np.int8)
         self._assert_setitem_series_conversion(obj, val, exp, exp_dtype)
 
     @pytest.mark.parametrize(
@@ -161,34 +168,19 @@ class TestSetitemCoercion(CoercionBase):
     @pytest.mark.parametrize(
         "val,exp_dtype",
         [
-            (1, np.int64),
-            (3, np.int64),
-            (1.1, np.float64),
-            (1 + 1j, np.complex128),
+            (1, object),
+            ("3", object),
+            (3, object),
+            (1.1, object),
+            (1 + 1j, object),
             (True, np.bool_),
         ],
     )
-    def test_setitem_series_bool(self, val, exp_dtype, request):
+    def test_setitem_series_bool(self, val, exp_dtype):
         obj = pd.Series([True, False, True, False])
         assert obj.dtype == np.bool_
 
-        mark = None
-        if exp_dtype is np.int64:
-            exp = pd.Series([True, True, True, False])
-            self._assert_setitem_series_conversion(obj, val, exp, np.bool_)
-            mark = pytest.mark.xfail(reason="TODO_GH12747 The result must be int")
-        elif exp_dtype is np.float64:
-            exp = pd.Series([True, True, True, False])
-            self._assert_setitem_series_conversion(obj, val, exp, np.bool_)
-            mark = pytest.mark.xfail(reason="TODO_GH12747 The result must be float")
-        elif exp_dtype is np.complex128:
-            exp = pd.Series([True, True, True, False])
-            self._assert_setitem_series_conversion(obj, val, exp, np.bool_)
-            mark = pytest.mark.xfail(reason="TODO_GH12747 The result must be complex")
-        if mark is not None:
-            request.node.add_marker(mark)
-
-        exp = pd.Series([True, val, True, False])
+        exp = pd.Series([True, val, True, False], dtype=exp_dtype)
         self._assert_setitem_series_conversion(obj, val, exp, exp_dtype)
 
     @pytest.mark.parametrize(
@@ -267,10 +259,19 @@ class TestSetitemCoercion(CoercionBase):
         )
         self._assert_setitem_series_conversion(obj, val, exp, exp_dtype)
 
+    def test_setitem_series_no_coercion_from_values_list(self):
+        # GH35865 - int casted to str when internally calling np.array(ser.values)
+        ser = pd.Series(["a", 1])
+        ser[:] = list(ser.values)
+
+        expected = pd.Series(["a", 1])
+
+        tm.assert_series_equal(ser, expected)
+
     def _assert_setitem_index_conversion(
         self, original_series, loc_key, expected_index, expected_dtype
     ):
-        """ test index's coercion triggered by assign key """
+        """test index's coercion triggered by assign key"""
         temp = original_series.copy()
         temp[loc_key] = 5
         exp = pd.Series([1, 2, 3, 4, 5], index=expected_index)
@@ -321,33 +322,41 @@ class TestSetitemCoercion(CoercionBase):
         if exp_dtype is IndexError:
             # float + int -> int
             temp = obj.copy()
-            with pytest.raises(exp_dtype):
+            msg = "index 5 is out of bounds for axis 0 with size 4"
+            with pytest.raises(exp_dtype, match=msg):
                 temp[5] = 5
             mark = pytest.mark.xfail(reason="TODO_GH12747 The result must be float")
             request.node.add_marker(mark)
         exp_index = pd.Index([1.1, 2.1, 3.1, 4.1, val])
         self._assert_setitem_index_conversion(obj, val, exp_index, exp_dtype)
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_setitem_series_period(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_setitem_index_complex128(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_setitem_index_bool(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_setitem_index_datetime64(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_setitem_index_datetime64tz(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_setitem_index_timedelta64(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_setitem_index_period(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
 
 class TestInsertIndexCoercion(CoercionBase):
@@ -356,7 +365,7 @@ class TestInsertIndexCoercion(CoercionBase):
     method = "insert"
 
     def _assert_insert_conversion(self, original, value, expected, expected_dtype):
-        """ test coercion triggered by insert """
+        """test coercion triggered by insert"""
         target = original.copy()
         res = target.insert(1, value)
         tm.assert_index_equal(res, expected)
@@ -383,7 +392,7 @@ class TestInsertIndexCoercion(CoercionBase):
         [
             (1, 1, np.int64),
             (1.1, 1.1, np.float64),
-            (False, 0, np.int64),
+            (False, False, object),  # GH#36319
             ("x", "x", object),
         ],
     )
@@ -399,7 +408,7 @@ class TestInsertIndexCoercion(CoercionBase):
         [
             (1, 1.0, np.float64),
             (1.1, 1.1, np.float64),
-            (False, 0.0, np.float64),
+            (False, False, object),  # GH#36319
             ("x", "x", object),
         ],
     )
@@ -418,7 +427,12 @@ class TestInsertIndexCoercion(CoercionBase):
         ],
         ids=["datetime64", "datetime64tz"],
     )
-    def test_insert_index_datetimes(self, fill_val, exp_dtype):
+    @pytest.mark.parametrize(
+        "insert_value",
+        [pd.Timestamp("2012-01-01"), pd.Timestamp("2012-01-01", tz="Asia/Tokyo"), 1],
+    )
+    def test_insert_index_datetimes(self, request, fill_val, exp_dtype, insert_value):
+
         obj = pd.DatetimeIndex(
             ["2011-01-01", "2011-01-02", "2011-01-03", "2011-01-04"], tz=fill_val.tz
         )
@@ -431,24 +445,35 @@ class TestInsertIndexCoercion(CoercionBase):
         self._assert_insert_conversion(obj, fill_val, exp, exp_dtype)
 
         if fill_val.tz:
-            msg = "Cannot compare tz-naive and tz-aware"
-            with pytest.raises(TypeError, match=msg):
-                obj.insert(1, pd.Timestamp("2012-01-01"))
 
-            msg = "Timezones don't match"
-            with pytest.raises(ValueError, match=msg):
-                obj.insert(1, pd.Timestamp("2012-01-01", tz="Asia/Tokyo"))
+            # mismatched tzawareness
+            ts = pd.Timestamp("2012-01-01")
+            result = obj.insert(1, ts)
+            expected = obj.astype(object).insert(1, ts)
+            assert expected.dtype == object
+            tm.assert_index_equal(result, expected)
+
+            # mismatched tz --> cast to object (could reasonably cast to common tz)
+            ts = pd.Timestamp("2012-01-01", tz="Asia/Tokyo")
+            result = obj.insert(1, ts)
+            expected = obj.astype(object).insert(1, ts)
+            assert expected.dtype == object
+            tm.assert_index_equal(result, expected)
 
         else:
-            msg = "Cannot compare tz-naive and tz-aware"
-            with pytest.raises(TypeError, match=msg):
-                obj.insert(1, pd.Timestamp("2012-01-01", tz="Asia/Tokyo"))
+            # mismatched tzawareness
+            ts = pd.Timestamp("2012-01-01", tz="Asia/Tokyo")
+            result = obj.insert(1, ts)
+            expected = obj.astype(object).insert(1, ts)
+            assert expected.dtype == object
+            tm.assert_index_equal(result, expected)
 
-        msg = "cannot insert DatetimeArray with incompatible label"
-        with pytest.raises(TypeError, match=msg):
-            obj.insert(1, 1)
-
-        pytest.xfail("ToDo: must coerce to object")
+        item = 1
+        result = obj.insert(1, item)
+        expected = obj.astype(object).insert(1, item)
+        assert expected[1] == item
+        assert expected.dtype == object
+        tm.assert_index_equal(result, expected)
 
     def test_insert_index_timedelta64(self):
         obj = pd.TimedeltaIndex(["1 day", "2 day", "3 day", "4 day"])
@@ -460,15 +485,11 @@ class TestInsertIndexCoercion(CoercionBase):
             obj, pd.Timedelta("10 day"), exp, "timedelta64[ns]"
         )
 
-        # ToDo: must coerce to object
-        msg = "cannot insert TimedeltaArray with incompatible label"
-        with pytest.raises(TypeError, match=msg):
-            obj.insert(1, pd.Timestamp("2012-01-01"))
-
-        # ToDo: must coerce to object
-        msg = "cannot insert TimedeltaArray with incompatible label"
-        with pytest.raises(TypeError, match=msg):
-            obj.insert(1, 1)
+        for item in [pd.Timestamp("2012-01-01"), 1]:
+            result = obj.insert(1, item)
+            expected = obj.astype(object).insert(1, item)
+            assert expected.dtype == object
+            tm.assert_index_equal(result, expected)
 
     @pytest.mark.parametrize(
         "insert, coerced_val, coerced_dtype",
@@ -493,16 +514,36 @@ class TestInsertIndexCoercion(CoercionBase):
         if isinstance(insert, pd.Period):
             exp = pd.PeriodIndex(data, freq="M")
             self._assert_insert_conversion(obj, insert, exp, coerced_dtype)
+
+            # string that can be parsed to appropriate PeriodDtype
+            self._assert_insert_conversion(obj, str(insert), exp, coerced_dtype)
+
         else:
+            result = obj.insert(0, insert)
+            expected = obj.astype(object).insert(0, insert)
+            tm.assert_index_equal(result, expected)
+
+            # TODO: ATM inserting '2012-01-01 00:00:00' when we have obj.freq=="M"
+            #  casts that string to Period[M], not clear that is desirable
+            if not isinstance(insert, pd.Timestamp):
+                # non-castable string
+                result = obj.insert(0, str(insert))
+                expected = obj.astype(object).insert(0, str(insert))
+                tm.assert_index_equal(result, expected)
+
             msg = r"Unexpected keyword arguments {'freq'}"
             with pytest.raises(TypeError, match=msg):
-                pd.Index(data, freq="M")
+                with tm.assert_produces_warning(FutureWarning):
+                    # passing keywords to pd.Index
+                    pd.Index(data, freq="M")
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_insert_index_complex128(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_insert_index_bool(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
 
 class TestWhereCoercion(CoercionBase):
@@ -512,10 +553,11 @@ class TestWhereCoercion(CoercionBase):
     def _assert_where_conversion(
         self, original, cond, values, expected, expected_dtype
     ):
-        """ test coercion triggered by where """
+        """test coercion triggered by where"""
         target = original.copy()
         res = target.where(cond, values)
-        self._assert(res, expected, expected_dtype)
+        tm.assert_equal(res, expected)
+        assert res.dtype == expected_dtype
 
     @pytest.mark.parametrize(
         "fill_val,exp_dtype",
@@ -538,7 +580,7 @@ class TestWhereCoercion(CoercionBase):
         if fill_val is True:
             values = klass([True, False, True, True])
         else:
-            values = klass(fill_val * x for x in [5, 6, 7, 8])
+            values = klass(x * fill_val for x in [5, 6, 7, 8])
 
         exp = klass(["a", values[1], "c", values[3]])
         self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
@@ -597,18 +639,19 @@ class TestWhereCoercion(CoercionBase):
         ],
     )
     def test_where_series_complex128(self, fill_val, exp_dtype):
-        obj = pd.Series([1 + 1j, 2 + 2j, 3 + 3j, 4 + 4j])
+        klass = pd.Series
+        obj = klass([1 + 1j, 2 + 2j, 3 + 3j, 4 + 4j])
         assert obj.dtype == np.complex128
-        cond = pd.Series([True, False, True, False])
+        cond = klass([True, False, True, False])
 
-        exp = pd.Series([1 + 1j, fill_val, 3 + 3j, fill_val])
+        exp = klass([1 + 1j, fill_val, 3 + 3j, fill_val])
         self._assert_where_conversion(obj, cond, fill_val, exp, exp_dtype)
 
         if fill_val is True:
-            values = pd.Series([True, False, True, True])
+            values = klass([True, False, True, True])
         else:
-            values = pd.Series(x * fill_val for x in [5, 6, 7, 8])
-        exp = pd.Series([1 + 1j, values[1], 3 + 3j, values[3]])
+            values = klass(x * fill_val for x in [5, 6, 7, 8])
+        exp = klass([1 + 1j, values[1], 3 + 3j, values[3]], dtype=exp_dtype)
         self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
 
     @pytest.mark.parametrize(
@@ -616,19 +659,20 @@ class TestWhereCoercion(CoercionBase):
         [(1, object), (1.1, object), (1 + 1j, object), (True, np.bool_)],
     )
     def test_where_series_bool(self, fill_val, exp_dtype):
+        klass = pd.Series
 
-        obj = pd.Series([True, False, True, False])
+        obj = klass([True, False, True, False])
         assert obj.dtype == np.bool_
-        cond = pd.Series([True, False, True, False])
+        cond = klass([True, False, True, False])
 
-        exp = pd.Series([True, fill_val, True, fill_val])
+        exp = klass([True, fill_val, True, fill_val])
         self._assert_where_conversion(obj, cond, fill_val, exp, exp_dtype)
 
         if fill_val is True:
-            values = pd.Series([True, False, True, True])
+            values = klass([True, False, True, True])
         else:
-            values = pd.Series(x * fill_val for x in [5, 6, 7, 8])
-        exp = pd.Series([True, values[1], True, values[3]])
+            values = klass(x * fill_val for x in [5, 6, 7, 8])
+        exp = klass([True, values[1], True, values[3]])
         self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
 
     @pytest.mark.parametrize(
@@ -747,17 +791,21 @@ class TestWhereCoercion(CoercionBase):
 
         self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_where_index_complex128(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_where_index_bool(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_where_series_timedelta64(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_where_series_period(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
     @pytest.mark.parametrize(
         "value", [pd.Timedelta(days=9), timedelta(days=9), np.timedelta64(9, "D")]
@@ -770,10 +818,13 @@ class TestWhereCoercion(CoercionBase):
         result = tdi.where(cond, value)
         tm.assert_index_equal(result, expected)
 
-        msg = "Where requires matching dtype"
-        with pytest.raises(TypeError, match=msg):
-            # wrong-dtyped NaT
-            tdi.where(cond, np.datetime64("NaT", "ns"))
+        # wrong-dtyped NaT
+        dtnat = np.datetime64("NaT", "ns")
+        expected = pd.Index([tdi[0], dtnat, dtnat, tdi[3]], dtype=object)
+        assert expected[1] is dtnat
+
+        result = tdi.where(cond, dtnat)
+        tm.assert_index_equal(result, expected)
 
     def test_where_index_period(self):
         dti = pd.date_range("2016-01-01", periods=3, freq="QS")
@@ -793,13 +844,16 @@ class TestWhereCoercion(CoercionBase):
         expected = pd.PeriodIndex([other[0], pi[1], other[2]])
         tm.assert_index_equal(result, expected)
 
-        # Passing a mismatched scalar
-        msg = "Where requires matching dtype"
-        with pytest.raises(TypeError, match=msg):
-            pi.where(cond, pd.Timedelta(days=4))
+        # Passing a mismatched scalar -> casts to object
+        td = pd.Timedelta(days=4)
+        expected = pd.Index([td, pi[1], td], dtype=object)
+        result = pi.where(cond, td)
+        tm.assert_index_equal(result, expected)
 
-        with pytest.raises(TypeError, match=msg):
-            pi.where(cond, pd.Period("2020-04-21", "D"))
+        per = pd.Period("2020-04-21", "D")
+        expected = pd.Index([per, pi[1], per], dtype=object)
+        result = pi.where(cond, per)
+        tm.assert_index_equal(result, expected)
 
 
 class TestFillnaSeriesCoercion(CoercionBase):
@@ -808,14 +862,16 @@ class TestFillnaSeriesCoercion(CoercionBase):
 
     method = "fillna"
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_has_comprehensive_tests(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
     def _assert_fillna_conversion(self, original, value, expected, expected_dtype):
-        """ test coercion triggered by fillna """
+        """test coercion triggered by fillna"""
         target = original.copy()
         res = target.fillna(value)
-        self._assert(res, expected, expected_dtype)
+        tm.assert_equal(res, expected)
+        assert res.dtype == expected_dtype
 
     @pytest.mark.parametrize(
         "fill_val, fill_dtype",
@@ -928,29 +984,37 @@ class TestFillnaSeriesCoercion(CoercionBase):
         )
         self._assert_fillna_conversion(obj, fill_val, exp, fill_dtype)
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_fillna_series_int64(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_fillna_index_int64(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_fillna_series_bool(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_fillna_index_bool(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_fillna_series_timedelta64(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_fillna_series_period(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_fillna_index_timedelta64(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_fillna_index_period(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
 
 
 class TestReplaceSeriesCoercion(CoercionBase):
@@ -958,7 +1022,7 @@ class TestReplaceSeriesCoercion(CoercionBase):
     klasses = ["series"]
     method = "replace"
 
-    rep: Dict[str, List] = {}
+    rep: dict[str, list] = {}
     rep["object"] = ["a", "b"]
     rep["int64"] = [4, 5]
     rep["float64"] = [1.1, 2.2]
@@ -976,10 +1040,28 @@ class TestReplaceSeriesCoercion(CoercionBase):
 
     rep["timedelta64[ns]"] = [pd.Timedelta("1 day"), pd.Timedelta("2 day")]
 
-    @pytest.mark.parametrize("how", ["dict", "series"])
-    @pytest.mark.parametrize(
-        "to_key",
-        [
+    @pytest.fixture(params=["dict", "series"])
+    def how(self, request):
+        return request.param
+
+    @pytest.fixture(
+        params=[
+            "object",
+            "int64",
+            "float64",
+            "complex128",
+            "bool",
+            "datetime64[ns]",
+            "datetime64[ns, UTC]",
+            "datetime64[ns, US/Eastern]",
+            "timedelta64[ns]",
+        ]
+    )
+    def from_key(self, request):
+        return request.param
+
+    @pytest.fixture(
+        params=[
             "object",
             "int64",
             "float64",
@@ -1002,21 +1084,23 @@ class TestReplaceSeriesCoercion(CoercionBase):
             "timedelta64",
         ],
     )
-    @pytest.mark.parametrize(
-        "from_key",
-        [
-            "object",
-            "int64",
-            "float64",
-            "complex128",
-            "bool",
-            "datetime64[ns]",
-            "datetime64[ns, UTC]",
-            "datetime64[ns, US/Eastern]",
-            "timedelta64[ns]",
-        ],
-    )
-    def test_replace_series(self, how, to_key, from_key):
+    def to_key(self, request):
+        return request.param
+
+    @pytest.fixture
+    def replacer(self, how, from_key, to_key):
+        """
+        Object we will pass to `Series.replace`
+        """
+        if how == "dict":
+            replacer = dict(zip(self.rep[from_key], self.rep[to_key]))
+        elif how == "series":
+            replacer = pd.Series(self.rep[to_key], index=self.rep[from_key])
+        else:
+            raise ValueError
+        return replacer
+
+    def test_replace_series(self, how, to_key, from_key, replacer):
         index = pd.Index([3, 4], name="xxx")
         obj = pd.Series(self.rep[from_key], index=index, name="yyy")
         assert obj.dtype == from_key
@@ -1028,20 +1112,13 @@ class TestReplaceSeriesCoercion(CoercionBase):
             # tested below
             return
 
-        if how == "dict":
-            replacer = dict(zip(self.rep[from_key], self.rep[to_key]))
-        elif how == "series":
-            replacer = pd.Series(self.rep[to_key], index=self.rep[from_key])
-        else:
-            raise ValueError
-
         result = obj.replace(replacer)
 
         if (from_key == "float64" and to_key in ("int64")) or (
             from_key == "complex128" and to_key in ("int64", "float64")
         ):
 
-            if compat.is_platform_32bit() or compat.is_platform_windows():
+            if not IS64 or is_platform_windows():
                 pytest.skip(f"32-bit platform buggy: {from_key} -> {to_key}")
 
             # Expected: do not downcast by replacement
@@ -1053,58 +1130,46 @@ class TestReplaceSeriesCoercion(CoercionBase):
 
         tm.assert_series_equal(result, exp)
 
-    @pytest.mark.parametrize("how", ["dict", "series"])
     @pytest.mark.parametrize(
         "to_key",
         ["timedelta64[ns]", "bool", "object", "complex128", "float64", "int64"],
+        indirect=True,
     )
     @pytest.mark.parametrize(
-        "from_key", ["datetime64[ns, UTC]", "datetime64[ns, US/Eastern]"]
+        "from_key", ["datetime64[ns, UTC]", "datetime64[ns, US/Eastern]"], indirect=True
     )
-    def test_replace_series_datetime_tz(self, how, to_key, from_key):
+    def test_replace_series_datetime_tz(self, how, to_key, from_key, replacer):
         index = pd.Index([3, 4], name="xyz")
         obj = pd.Series(self.rep[from_key], index=index, name="yyy")
         assert obj.dtype == from_key
 
-        if how == "dict":
-            replacer = dict(zip(self.rep[from_key], self.rep[to_key]))
-        elif how == "series":
-            replacer = pd.Series(self.rep[to_key], index=self.rep[from_key])
-        else:
-            raise ValueError
-
         result = obj.replace(replacer)
         exp = pd.Series(self.rep[to_key], index=index, name="yyy")
         assert exp.dtype == to_key
 
         tm.assert_series_equal(result, exp)
 
-    @pytest.mark.parametrize("how", ["dict", "series"])
     @pytest.mark.parametrize(
         "to_key",
         ["datetime64[ns]", "datetime64[ns, UTC]", "datetime64[ns, US/Eastern]"],
+        indirect=True,
     )
     @pytest.mark.parametrize(
         "from_key",
         ["datetime64[ns]", "datetime64[ns, UTC]", "datetime64[ns, US/Eastern]"],
+        indirect=True,
     )
-    def test_replace_series_datetime_datetime(self, how, to_key, from_key):
+    def test_replace_series_datetime_datetime(self, how, to_key, from_key, replacer):
         index = pd.Index([3, 4], name="xyz")
         obj = pd.Series(self.rep[from_key], index=index, name="yyy")
         assert obj.dtype == from_key
 
-        if how == "dict":
-            replacer = dict(zip(self.rep[from_key], self.rep[to_key]))
-        elif how == "series":
-            replacer = pd.Series(self.rep[to_key], index=self.rep[from_key])
-        else:
-            raise ValueError
-
         result = obj.replace(replacer)
         exp = pd.Series(self.rep[to_key], index=index, name="yyy")
         assert exp.dtype == to_key
 
         tm.assert_series_equal(result, exp)
 
+    @pytest.mark.xfail(reason="Test not implemented")
     def test_replace_series_period(self):
-        pytest.xfail("Test not implemented")
+        raise NotImplementedError
