@@ -1,4 +1,4 @@
-from collections import OrderedDict
+import re
 
 import numpy as np
 import pytest
@@ -6,9 +6,18 @@ import pytest
 from pandas.core.dtypes.common import is_integer_dtype
 
 import pandas as pd
-from pandas import Categorical, CategoricalIndex, DataFrame, Series, get_dummies
+from pandas import (
+    Categorical,
+    CategoricalIndex,
+    DataFrame,
+    Series,
+    get_dummies,
+)
 import pandas._testing as tm
-from pandas.core.arrays.sparse import SparseArray, SparseDtype
+from pandas.core.arrays.sparse import (
+    SparseArray,
+    SparseDtype,
+)
 
 
 class TestGetDummies:
@@ -32,7 +41,8 @@ class TestGetDummies:
         return dtype
 
     def test_get_dummies_raises_on_dtype_object(self, df):
-        with pytest.raises(ValueError):
+        msg = "dtype=object is not a valid dtype for get_dummies"
+        with pytest.raises(ValueError, match=msg):
             get_dummies(df, dtype="object")
 
     def test_get_dummies_basic(self, sparse, dtype):
@@ -161,7 +171,7 @@ class TestGetDummies:
         s = [e, eacute, eacute]
         res = get_dummies(s, prefix="letter", sparse=sparse)
         exp = DataFrame(
-            {"letter_e": [1, 0, 0], f"letter_{eacute}": [0, 1, 1]}, dtype=np.uint8,
+            {"letter_e": [1, 0, 0], f"letter_{eacute}": [0, 1, 1]}, dtype=np.uint8
         )
         if sparse:
             exp = exp.apply(SparseArray, fill_value=0)
@@ -262,8 +272,9 @@ class TestGetDummies:
                 "from_A_a": [1, 0, 1],
                 "from_A_b": [0, 1, 0],
             },
-            dtype=np.uint8,
         )
+        cols = expected.columns
+        expected[cols[1:]] = expected[cols[1:]].astype(np.uint8)
         expected[["C"]] = df[["C"]]
         if sparse:
             cols = ["from_A_a", "from_A_b"]
@@ -298,11 +309,19 @@ class TestGetDummies:
         tm.assert_frame_equal(result, expected)
 
     def test_dataframe_dummies_prefix_bad_length(self, df, sparse):
-        with pytest.raises(ValueError):
+        msg = re.escape(
+            "Length of 'prefix' (1) did not match the length of the columns being "
+            "encoded (2)"
+        )
+        with pytest.raises(ValueError, match=msg):
             get_dummies(df, prefix=["too few"], sparse=sparse)
 
     def test_dataframe_dummies_prefix_sep_bad_length(self, df, sparse):
-        with pytest.raises(ValueError):
+        msg = re.escape(
+            "Length of 'prefix_sep' (1) did not match the length of the columns being "
+            "encoded (2)"
+        )
+        with pytest.raises(ValueError, match=msg):
             get_dummies(df, prefix_sep=["bad"], sparse=sparse)
 
     def test_dataframe_dummies_prefix_dict(self, sparse):
@@ -386,7 +405,7 @@ class TestGetDummies:
         "get_dummies_kwargs,expected",
         [
             (
-                {"data": DataFrame(({"ä": ["a"]}))},
+                {"data": DataFrame({"ä": ["a"]})},
                 DataFrame({"ä_a": [1]}, dtype=np.uint8),
             ),
             (
@@ -569,9 +588,7 @@ class TestGetDummies:
     @pytest.mark.parametrize("sparse", [True, False])
     def test_get_dummies_dont_sparsify_all_columns(self, sparse):
         # GH18914
-        df = DataFrame.from_dict(
-            OrderedDict([("GDP", [1, 2]), ("Nation", ["AB", "CD"])])
-        )
+        df = DataFrame.from_dict({"GDP": [1, 2], "Nation": ["AB", "CD"]})
         df = get_dummies(df, columns=["Nation"], sparse=sparse)
         df2 = df.reindex(columns=["GDP"])
 
