@@ -1,26 +1,18 @@
 from datetime import datetime
-import warnings
 
 import numpy as np
-from numpy.random import randn
 import pytest
 
 import pandas.util._test_decorators as td
 
-import pandas as pd
-from pandas import DataFrame, DatetimeIndex, Index, Series
-import pandas._testing as tm
-from pandas.core.window.common import _flex_binary_moment
-from pandas.tests.window.common import (
-    check_pairwise_moment,
-    moments_consistency_cov_data,
-    moments_consistency_is_constant,
-    moments_consistency_mock_mean,
-    moments_consistency_series_data,
-    moments_consistency_std_data,
-    moments_consistency_var_data,
-    moments_consistency_var_debiasing_factors,
+from pandas import (
+    DataFrame,
+    DatetimeIndex,
+    Index,
+    MultiIndex,
+    Series,
 )
+import pandas._testing as tm
 
 
 def _rolling_consistency_cases():
@@ -35,7 +27,7 @@ def _rolling_consistency_cases():
 # binary moments
 def test_rolling_cov(series):
     A = series
-    B = A + randn(len(A))
+    B = A + np.random.randn(len(A))
 
     result = A.rolling(window=50, min_periods=25).cov(B)
     tm.assert_almost_equal(result[-1], np.cov(A[-50:], B[-50:])[0, 1])
@@ -43,7 +35,7 @@ def test_rolling_cov(series):
 
 def test_rolling_corr(series):
     A = series
-    B = A + randn(len(A))
+    B = A + np.random.randn(len(A))
 
     result = A.rolling(window=50, min_periods=25).corr(B)
     tm.assert_almost_equal(result[-1], np.corrcoef(A[-50:], B[-50:])[0, 1])
@@ -60,7 +52,11 @@ def test_rolling_corr(series):
 
 @pytest.mark.parametrize("func", ["cov", "corr"])
 def test_rolling_pairwise_cov_corr(func, frame):
-    check_pairwise_moment(frame, "rolling", func, window=10, min_periods=5)
+    result = getattr(frame.rolling(window=10, min_periods=5), func)()
+    result = result.loc[(slice(None), 1), 5]
+    result.index = result.index.droplevel(1)
+    expected = getattr(frame[1].rolling(window=10, min_periods=5), func)(frame[5])
+    tm.assert_series_equal(result, expected, check_names=False)
 
 
 @pytest.mark.parametrize("method", ["corr", "cov"])
@@ -84,75 +80,58 @@ def test_flex_binary_frame(method, frame):
     tm.assert_frame_equal(res3, exp)
 
 
-@pytest.mark.slow
 @pytest.mark.parametrize(
     "window,min_periods,center", list(_rolling_consistency_cases())
 )
-def test_rolling_apply_consistency(
-    consistency_data, base_functions, no_nan_functions, window, min_periods, center
+@pytest.mark.parametrize("f", [lambda v: Series(v).sum(), np.nansum])
+def test_rolling_apply_consistency_sum_nans(
+    consistency_data, window, min_periods, center, f
 ):
     x, is_constant, no_nans = consistency_data
 
-    with warnings.catch_warnings():
-        warnings.filterwarnings(
-            "ignore", message=".*(empty slice|0 for slice).*", category=RuntimeWarning,
-        )
-        # test consistency between rolling_xyz() and either (a)
-        # rolling_apply of Series.xyz(), or (b) rolling_apply of
-        # np.nanxyz()
-        functions = base_functions
+    if f is np.nansum and min_periods == 0:
+        pass
+    else:
+        rolling_f_result = x.rolling(
+            window=window, min_periods=min_periods, center=center
+        ).sum()
+        rolling_apply_f_result = x.rolling(
+            window=window, min_periods=min_periods, center=center
+        ).apply(func=f, raw=True)
+        tm.assert_equal(rolling_f_result, rolling_apply_f_result)
 
-        # GH 8269
-        if no_nans:
-            functions = no_nan_functions + base_functions
-        for (f, require_min_periods, name) in functions:
-            rolling_f = getattr(
-                x.rolling(window=window, center=center, min_periods=min_periods), name,
-            )
 
-            if (
-                require_min_periods
-                and (min_periods is not None)
-                and (min_periods < require_min_periods)
-            ):
-                continue
+@pytest.mark.parametrize(
+    "window,min_periods,center", list(_rolling_consistency_cases())
+)
+@pytest.mark.parametrize("f", [lambda v: Series(v).sum(), np.nansum, np.sum])
+def test_rolling_apply_consistency_sum_no_nans(
+    consistency_data, window, min_periods, center, f
+):
+    x, is_constant, no_nans = consistency_data
 
-            if name == "count":
-                rolling_f_result = rolling_f()
-                rolling_apply_f_result = x.rolling(
-                    window=window, min_periods=min_periods, center=center
-                ).apply(func=f, raw=True)
-            else:
-                if name in ["cov", "corr"]:
-                    rolling_f_result = rolling_f(pairwise=False)
-                else:
-                    rolling_f_result = rolling_f()
-                rolling_apply_f_result = x.rolling(
-                    window=window, min_periods=min_periods, center=center
-                ).apply(func=f, raw=True)
-
-            # GH 9422
-            if name in ["sum", "prod"]:
-                tm.assert_equal(rolling_f_result, rolling_apply_f_result)
+    if no_nans:
+        if f is np.nansum and min_periods == 0:
+            pass
+        else:
+            rolling_f_result = x.rolling(
+                window=window, min_periods=min_periods, center=center
+            ).sum()
+            rolling_apply_f_result = x.rolling(
+                window=window, min_periods=min_periods, center=center
+            ).apply(func=f, raw=True)
+            tm.assert_equal(rolling_f_result, rolling_apply_f_result)
 
 
 @pytest.mark.parametrize("window", range(7))
 def test_rolling_corr_with_zero_variance(window):
     # GH 18430
-    s = pd.Series(np.zeros(20))
-    other = pd.Series(np.arange(20))
+    s = Series(np.zeros(20))
+    other = Series(np.arange(20))
 
     assert s.rolling(window=window).corr(other=other).isna().all()
 
 
-def test_flex_binary_moment():
-    # GH3155
-    # don't blow the stack
-    msg = "arguments to moment function must be of type np.ndarray/Series/DataFrame"
-    with pytest.raises(TypeError, match=msg):
-        _flex_binary_moment(5, 6, None)
-
-
 def test_corr_sanity():
     # GH 3155
     df = DataFrame(
@@ -171,14 +150,9 @@ def test_corr_sanity():
     res = df[0].rolling(5, center=True).corr(df[1])
     assert all(np.abs(np.nan_to_num(x)) <= 1 for x in res)
 
-    # and some fuzzing
-    for _ in range(10):
-        df = DataFrame(np.random.rand(30, 2))
-        res = df[0].rolling(5, center=True).corr(df[1])
-        try:
-            assert all(np.abs(np.nan_to_num(x)) <= 1 for x in res)
-        except AssertionError:
-            print(res)
+    df = DataFrame(np.random.rand(30, 2))
+    res = df[0].rolling(5, center=True).corr(df[1])
+    assert all(np.abs(np.nan_to_num(x)) <= 1 for x in res)
 
 
 def test_rolling_cov_diff_length():
@@ -224,10 +198,12 @@ def test_rolling_corr_diff_length():
         lambda x: x.rolling(window=10, min_periods=5).median(),
         lambda x: x.rolling(window=10, min_periods=5).apply(sum, raw=False),
         lambda x: x.rolling(window=10, min_periods=5).apply(sum, raw=True),
-        lambda x: x.rolling(win_type="boxcar", window=10, min_periods=5).mean(),
+        pytest.param(
+            lambda x: x.rolling(win_type="boxcar", window=10, min_periods=5).mean(),
+            marks=td.skip_if_no_scipy,
+        ),
     ],
 )
-@td.skip_if_no_scipy
 def test_rolling_functions_window_non_shrinkage(f):
     # GH 7764
     s = Series(range(4))
@@ -242,7 +218,14 @@ def test_rolling_functions_window_non_shrinkage(f):
     tm.assert_frame_equal(df_result, df_expected)
 
 
-def test_rolling_functions_window_non_shrinkage_binary():
+@pytest.mark.parametrize(
+    "f",
+    [
+        lambda x: (x.rolling(window=10, min_periods=5).cov(x, pairwise=True)),
+        lambda x: (x.rolling(window=10, min_periods=5).corr(x, pairwise=True)),
+    ],
+)
+def test_rolling_functions_window_non_shrinkage_binary(f):
 
     # corr/cov return a MI DataFrame
     df = DataFrame(
@@ -252,16 +235,11 @@ def test_rolling_functions_window_non_shrinkage_binary():
     )
     df_expected = DataFrame(
         columns=Index(["A", "B"], name="foo"),
-        index=pd.MultiIndex.from_product([df.index, df.columns], names=["bar", "foo"]),
+        index=MultiIndex.from_product([df.index, df.columns], names=["bar", "foo"]),
         dtype="float64",
     )
-    functions = [
-        lambda x: (x.rolling(window=10, min_periods=5).cov(x, pairwise=True)),
-        lambda x: (x.rolling(window=10, min_periods=5).corr(x, pairwise=True)),
-    ]
-    for f in functions:
-        df_result = f(df)
-        tm.assert_frame_equal(df_result, df_expected)
+    df_result = f(df)
+    tm.assert_frame_equal(df_result, df_expected)
 
 
 def test_rolling_skew_edge_cases():
@@ -424,35 +402,27 @@ def test_rolling_median_memory_error():
     Series(np.random.randn(n)).rolling(window=2, center=False).median()
 
 
-def test_rolling_min_max_numeric_types():
-
+@pytest.mark.parametrize(
+    "data_type",
+    [np.dtype(f"f{width}") for width in [4, 8]]
+    + [np.dtype(f"{sign}{width}") for width in [1, 2, 4, 8] for sign in "ui"],
+)
+def test_rolling_min_max_numeric_types(data_type):
     # GH12373
-    types_test = [np.dtype(f"f{width}") for width in [4, 8]]
-    types_test.extend(
-        [np.dtype(f"{sign}{width}") for width in [1, 2, 4, 8] for sign in "ui"]
-    )
-    for data_type in types_test:
-        # Just testing that these don't throw exceptions and that
-        # the return type is float64. Other tests will cover quantitative
-        # correctness
-        result = DataFrame(np.arange(20, dtype=data_type)).rolling(window=5).max()
-        assert result.dtypes[0] == np.dtype("f8")
-        result = DataFrame(np.arange(20, dtype=data_type)).rolling(window=5).min()
-        assert result.dtypes[0] == np.dtype("f8")
+
+    # Just testing that these don't throw exceptions and that
+    # the return type is float64. Other tests will cover quantitative
+    # correctness
+    result = DataFrame(np.arange(20, dtype=data_type)).rolling(window=5).max()
+    assert result.dtypes[0] == np.dtype("f8")
+    result = DataFrame(np.arange(20, dtype=data_type)).rolling(window=5).min()
+    assert result.dtypes[0] == np.dtype("f8")
 
 
-def test_moment_functions_zero_length():
-    # GH 8056
-    s = Series(dtype=np.float64)
-    s_expected = s
-    df1 = DataFrame()
-    df1_expected = df1
-    df2 = DataFrame(columns=["a"])
-    df2["a"] = df2["a"].astype("float64")
-    df2_expected = df2
-
-    functions = [
-        lambda x: x.rolling(window=10).count(),
+@pytest.mark.parametrize(
+    "f",
+    [
+        lambda x: x.rolling(window=10, min_periods=0).count(),
         lambda x: x.rolling(window=10, min_periods=5).cov(x, pairwise=False),
         lambda x: x.rolling(window=10, min_periods=5).corr(x, pairwise=False),
         lambda x: x.rolling(window=10, min_periods=5).max(),
@@ -467,229 +437,276 @@ def test_moment_functions_zero_length():
         lambda x: x.rolling(window=10, min_periods=5).median(),
         lambda x: x.rolling(window=10, min_periods=5).apply(sum, raw=False),
         lambda x: x.rolling(window=10, min_periods=5).apply(sum, raw=True),
-        lambda x: x.rolling(win_type="boxcar", window=10, min_periods=5).mean(),
-    ]
-    for f in functions:
-        try:
-            s_result = f(s)
-            tm.assert_series_equal(s_result, s_expected)
+        pytest.param(
+            lambda x: x.rolling(win_type="boxcar", window=10, min_periods=5).mean(),
+            marks=td.skip_if_no_scipy,
+        ),
+    ],
+)
+def test_moment_functions_zero_length(f):
+    # GH 8056
+    s = Series(dtype=np.float64)
+    s_expected = s
+    df1 = DataFrame()
+    df1_expected = df1
+    df2 = DataFrame(columns=["a"])
+    df2["a"] = df2["a"].astype("float64")
+    df2_expected = df2
 
-            df1_result = f(df1)
-            tm.assert_frame_equal(df1_result, df1_expected)
+    s_result = f(s)
+    tm.assert_series_equal(s_result, s_expected)
 
-            df2_result = f(df2)
-            tm.assert_frame_equal(df2_result, df2_expected)
-        except (ImportError):
+    df1_result = f(df1)
+    tm.assert_frame_equal(df1_result, df1_expected)
 
-            # scipy needed for rolling_window
-            continue
+    df2_result = f(df2)
+    tm.assert_frame_equal(df2_result, df2_expected)
 
 
-def test_moment_functions_zero_length_pairwise():
+@pytest.mark.parametrize(
+    "f",
+    [
+        lambda x: (x.rolling(window=10, min_periods=5).cov(x, pairwise=True)),
+        lambda x: (x.rolling(window=10, min_periods=5).corr(x, pairwise=True)),
+    ],
+)
+def test_moment_functions_zero_length_pairwise(f):
 
     df1 = DataFrame()
     df2 = DataFrame(columns=Index(["a"], name="foo"), index=Index([], name="bar"))
     df2["a"] = df2["a"].astype("float64")
 
     df1_expected = DataFrame(
-        index=pd.MultiIndex.from_product([df1.index, df1.columns]), columns=Index([]),
+        index=MultiIndex.from_product([df1.index, df1.columns]), columns=Index([])
     )
     df2_expected = DataFrame(
-        index=pd.MultiIndex.from_product(
-            [df2.index, df2.columns], names=["bar", "foo"]
-        ),
+        index=MultiIndex.from_product([df2.index, df2.columns], names=["bar", "foo"]),
         columns=Index(["a"], name="foo"),
         dtype="float64",
     )
 
-    functions = [
-        lambda x: (x.rolling(window=10, min_periods=5).cov(x, pairwise=True)),
-        lambda x: (x.rolling(window=10, min_periods=5).corr(x, pairwise=True)),
-    ]
+    df1_result = f(df1)
+    tm.assert_frame_equal(df1_result, df1_expected)
 
-    for f in functions:
-        df1_result = f(df1)
-        tm.assert_frame_equal(df1_result, df1_expected)
-
-        df2_result = f(df2)
-        tm.assert_frame_equal(df2_result, df2_expected)
+    df2_result = f(df2)
+    tm.assert_frame_equal(df2_result, df2_expected)
 
 
-@pytest.mark.slow
 @pytest.mark.parametrize(
     "window,min_periods,center", list(_rolling_consistency_cases())
 )
-def test_rolling_consistency_var(consistency_data, window, min_periods, center):
+@pytest.mark.parametrize("ddof", [0, 1])
+def test_moments_consistency_var(consistency_data, window, min_periods, center, ddof):
     x, is_constant, no_nans = consistency_data
-    moments_consistency_var_data(
-        x=x,
-        is_constant=is_constant,
-        min_periods=min_periods,
-        count=lambda x: (
+
+    mean_x = x.rolling(window=window, min_periods=min_periods, center=center).mean()
+    var_x = x.rolling(window=window, min_periods=min_periods, center=center).var(
+        ddof=ddof
+    )
+    assert not (var_x < 0).any().any()
+
+    if ddof == 0:
+        # check that biased var(x) == mean(x^2) - mean(x)^2
+        mean_x2 = (
+            (x * x)
+            .rolling(window=window, min_periods=min_periods, center=center)
+            .mean()
+        )
+        tm.assert_equal(var_x, mean_x2 - (mean_x * mean_x))
+
+
+@pytest.mark.parametrize(
+    "window,min_periods,center", list(_rolling_consistency_cases())
+)
+@pytest.mark.parametrize("ddof", [0, 1])
+def test_moments_consistency_var_constant(
+    consistency_data, window, min_periods, center, ddof
+):
+    x, is_constant, no_nans = consistency_data
+
+    if is_constant:
+        count_x = x.rolling(
+            window=window, min_periods=min_periods, center=center
+        ).count()
+        var_x = x.rolling(window=window, min_periods=min_periods, center=center).var(
+            ddof=ddof
+        )
+
+        # check that variance of constant series is identically 0
+        assert not (var_x > 0).any().any()
+        expected = x * np.nan
+        expected[count_x >= max(min_periods, 1)] = 0.0
+        if ddof == 1:
+            expected[count_x < 2] = np.nan
+        tm.assert_equal(var_x, expected)
+
+
+@pytest.mark.parametrize(
+    "window,min_periods,center", list(_rolling_consistency_cases())
+)
+@pytest.mark.parametrize("ddof", [0, 1])
+def test_rolling_consistency_std(consistency_data, window, min_periods, center, ddof):
+    x, is_constant, no_nans = consistency_data
+
+    var_x = x.rolling(window=window, min_periods=min_periods, center=center).var(
+        ddof=ddof
+    )
+    std_x = x.rolling(window=window, min_periods=min_periods, center=center).std(
+        ddof=ddof
+    )
+    assert not (var_x < 0).any().any()
+    assert not (std_x < 0).any().any()
+
+    # check that var(x) == std(x)^2
+    tm.assert_equal(var_x, std_x * std_x)
+
+
+@pytest.mark.parametrize(
+    "window,min_periods,center", list(_rolling_consistency_cases())
+)
+@pytest.mark.parametrize("ddof", [0, 1])
+def test_rolling_consistency_cov(consistency_data, window, min_periods, center, ddof):
+    x, is_constant, no_nans = consistency_data
+    var_x = x.rolling(window=window, min_periods=min_periods, center=center).var(
+        ddof=ddof
+    )
+    assert not (var_x < 0).any().any()
+
+    cov_x_x = x.rolling(window=window, min_periods=min_periods, center=center).cov(
+        x, ddof=ddof
+    )
+    assert not (cov_x_x < 0).any().any()
+
+    # check that var(x) == cov(x, x)
+    tm.assert_equal(var_x, cov_x_x)
+
+
+@pytest.mark.parametrize(
+    "window,min_periods,center", list(_rolling_consistency_cases())
+)
+@pytest.mark.parametrize("ddof", [0, 1])
+def test_rolling_consistency_series_cov_corr(
+    consistency_data, window, min_periods, center, ddof
+):
+    x, is_constant, no_nans = consistency_data
+
+    if isinstance(x, Series):
+        var_x_plus_y = (
+            (x + x)
+            .rolling(window=window, min_periods=min_periods, center=center)
+            .var(ddof=ddof)
+        )
+        var_x = x.rolling(window=window, min_periods=min_periods, center=center).var(
+            ddof=ddof
+        )
+        var_y = x.rolling(window=window, min_periods=min_periods, center=center).var(
+            ddof=ddof
+        )
+        cov_x_y = x.rolling(window=window, min_periods=min_periods, center=center).cov(
+            x, ddof=ddof
+        )
+        # check that cov(x, y) == (var(x+y) - var(x) -
+        # var(y)) / 2
+        tm.assert_equal(cov_x_y, 0.5 * (var_x_plus_y - var_x - var_y))
+
+        # check that corr(x, y) == cov(x, y) / (std(x) *
+        # std(y))
+        corr_x_y = x.rolling(
+            window=window, min_periods=min_periods, center=center
+        ).corr(x)
+        std_x = x.rolling(window=window, min_periods=min_periods, center=center).std(
+            ddof=ddof
+        )
+        std_y = x.rolling(window=window, min_periods=min_periods, center=center).std(
+            ddof=ddof
+        )
+        tm.assert_equal(corr_x_y, cov_x_y / (std_x * std_y))
+
+        if ddof == 0:
+            # check that biased cov(x, y) == mean(x*y) -
+            # mean(x)*mean(y)
+            mean_x = x.rolling(
+                window=window, min_periods=min_periods, center=center
+            ).mean()
+            mean_y = x.rolling(
+                window=window, min_periods=min_periods, center=center
+            ).mean()
+            mean_x_times_y = (
+                (x * x)
+                .rolling(window=window, min_periods=min_periods, center=center)
+                .mean()
+            )
+            tm.assert_equal(cov_x_y, mean_x_times_y - (mean_x * mean_y))
+
+
+@pytest.mark.parametrize(
+    "window,min_periods,center", list(_rolling_consistency_cases())
+)
+def test_rolling_consistency_mean(consistency_data, window, min_periods, center):
+    x, is_constant, no_nans = consistency_data
+
+    result = x.rolling(window=window, min_periods=min_periods, center=center).mean()
+    expected = (
+        x.rolling(window=window, min_periods=min_periods, center=center)
+        .sum()
+        .divide(
             x.rolling(window=window, min_periods=min_periods, center=center).count()
-        ),
-        mean=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).mean()
-        ),
-        var_unbiased=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).var()
-        ),
-        var_biased=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).var(ddof=0)
-        ),
-    )
-
-
-@pytest.mark.slow
-@pytest.mark.parametrize(
-    "window,min_periods,center", list(_rolling_consistency_cases())
-)
-def test_rolling_consistency_std(consistency_data, window, min_periods, center):
-    x, is_constant, no_nans = consistency_data
-    moments_consistency_std_data(
-        x=x,
-        var_unbiased=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).var()
-        ),
-        std_unbiased=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).std()
-        ),
-        var_biased=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).var(ddof=0)
-        ),
-        std_biased=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).std(ddof=0)
-        ),
-    )
-
-
-@pytest.mark.slow
-@pytest.mark.parametrize(
-    "window,min_periods,center", list(_rolling_consistency_cases())
-)
-def test_rolling_consistency_cov(consistency_data, window, min_periods, center):
-    x, is_constant, no_nans = consistency_data
-    moments_consistency_cov_data(
-        x=x,
-        var_unbiased=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).var()
-        ),
-        cov_unbiased=lambda x, y: (
-            x.rolling(window=window, min_periods=min_periods, center=center).cov(y)
-        ),
-        var_biased=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).var(ddof=0)
-        ),
-        cov_biased=lambda x, y: (
-            x.rolling(window=window, min_periods=min_periods, center=center).cov(
-                y, ddof=0
-            )
-        ),
-    )
-
-
-@pytest.mark.slow
-@pytest.mark.parametrize(
-    "window,min_periods,center", list(_rolling_consistency_cases())
-)
-def test_rolling_consistency_series(consistency_data, window, min_periods, center):
-    x, is_constant, no_nans = consistency_data
-    moments_consistency_series_data(
-        x=x,
-        mean=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).mean()
-        ),
-        corr=lambda x, y: (
-            x.rolling(window=window, min_periods=min_periods, center=center).corr(y)
-        ),
-        var_unbiased=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).var()
-        ),
-        std_unbiased=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).std()
-        ),
-        cov_unbiased=lambda x, y: (
-            x.rolling(window=window, min_periods=min_periods, center=center).cov(y)
-        ),
-        var_biased=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).var(ddof=0)
-        ),
-        std_biased=lambda x: (
-            x.rolling(window=window, min_periods=min_periods, center=center).std(ddof=0)
-        ),
-        cov_biased=lambda x, y: (
-            x.rolling(window=window, min_periods=min_periods, center=center).cov(
-                y, ddof=0
-            )
-        ),
-    )
-
-
-@pytest.mark.slow
-@pytest.mark.parametrize(
-    "window,min_periods,center", list(_rolling_consistency_cases())
-)
-def test_rolling_consistency(consistency_data, window, min_periods, center):
-    x, is_constant, no_nans = consistency_data
-    # suppress warnings about empty slices, as we are deliberately testing
-    # with empty/0-length Series/DataFrames
-    with warnings.catch_warnings():
-        warnings.filterwarnings(
-            "ignore", message=".*(empty slice|0 for slice).*", category=RuntimeWarning,
         )
+    )
+    tm.assert_equal(result, expected.astype("float64"))
 
-        # test consistency between different rolling_* moments
-        moments_consistency_mock_mean(
-            x=x,
-            mean=lambda x: (
-                x.rolling(window=window, min_periods=min_periods, center=center).mean()
-            ),
-            mock_mean=lambda x: (
-                x.rolling(window=window, min_periods=min_periods, center=center)
-                .sum()
-                .divide(
-                    x.rolling(
-                        window=window, min_periods=min_periods, center=center
-                    ).count()
-                )
-            ),
-        )
 
-        moments_consistency_is_constant(
-            x=x,
-            is_constant=is_constant,
-            min_periods=min_periods,
-            count=lambda x: (
+@pytest.mark.parametrize(
+    "window,min_periods,center", list(_rolling_consistency_cases())
+)
+def test_rolling_consistency_constant(consistency_data, window, min_periods, center):
+    x, is_constant, no_nans = consistency_data
+
+    if is_constant:
+        count_x = x.rolling(
+            window=window, min_periods=min_periods, center=center
+        ).count()
+        mean_x = x.rolling(window=window, min_periods=min_periods, center=center).mean()
+        # check that correlation of a series with itself is either 1 or NaN
+        corr_x_x = x.rolling(
+            window=window, min_periods=min_periods, center=center
+        ).corr(x)
+
+        exp = x.max() if isinstance(x, Series) else x.max().max()
+
+        # check mean of constant series
+        expected = x * np.nan
+        expected[count_x >= max(min_periods, 1)] = exp
+        tm.assert_equal(mean_x, expected)
+
+        # check correlation of constant series with itself is NaN
+        expected[:] = np.nan
+        tm.assert_equal(corr_x_x, expected)
+
+
+@pytest.mark.parametrize(
+    "window,min_periods,center", list(_rolling_consistency_cases())
+)
+def test_rolling_consistency_var_debiasing_factors(
+    consistency_data, window, min_periods, center
+):
+    x, is_constant, no_nans = consistency_data
+
+    # check variance debiasing factors
+    var_unbiased_x = x.rolling(
+        window=window, min_periods=min_periods, center=center
+    ).var()
+    var_biased_x = x.rolling(window=window, min_periods=min_periods, center=center).var(
+        ddof=0
+    )
+    var_debiasing_factors_x = (
+        x.rolling(window=window, min_periods=min_periods, center=center)
+        .count()
+        .divide(
+            (
                 x.rolling(window=window, min_periods=min_periods, center=center).count()
-            ),
-            mean=lambda x: (
-                x.rolling(window=window, min_periods=min_periods, center=center).mean()
-            ),
-            corr=lambda x, y: (
-                x.rolling(window=window, min_periods=min_periods, center=center).corr(y)
-            ),
-        )
-
-        moments_consistency_var_debiasing_factors(
-            x=x,
-            var_unbiased=lambda x: (
-                x.rolling(window=window, min_periods=min_periods, center=center).var()
-            ),
-            var_biased=lambda x: (
-                x.rolling(window=window, min_periods=min_periods, center=center).var(
-                    ddof=0
-                )
-            ),
-            var_debiasing_factors=lambda x: (
-                x.rolling(window=window, min_periods=min_periods, center=center)
-                .count()
-                .divide(
-                    (
-                        x.rolling(
-                            window=window, min_periods=min_periods, center=center
-                        ).count()
-                        - 1.0
-                    ).replace(0.0, np.nan)
-                )
-            ),
+                - 1.0
+            ).replace(0.0, np.nan)
         )
+    )
+    tm.assert_equal(var_unbiased_x, var_biased_x * var_debiasing_factors_x)
